@@ -1,4 +1,4 @@
-from dataclasses import is_dataclass
+from dataclasses import is_dataclass, fields as dataclass_fields
 from enum import Enum
 import sys
 import types
@@ -76,8 +76,10 @@ def _replace_from_dict(cls, from_dict='from_dict'):
     the_globals = {
         # use the defining modules globals
         **sys.modules[cls.__module__].__dict__,
+        # along with any decoders
+        **decoders(cls),
         # along with types we use for the conversion
-        **referenced_types(cls)
+        **referenced_types(cls),
     }
 
     from_dict_func = types.FunctionType(
@@ -100,9 +102,16 @@ def _replace_to_dict(cls, to_dict='to_dict'):
         if isinstance(const, types.CodeType)
     ][0]
 
+    the_globals = {
+        # use the defining modules globals
+        **sys.modules[cls.__module__].__dict__,
+        # along with any encoders
+        **encoders(cls),
+    }
+
     to_dict_func = types.FunctionType(
         to_dict_code,
-        sys.modules[cls.__module__].__dict__,
+        the_globals,
         to_dict,
     )
 
@@ -115,15 +124,31 @@ def _from_dict_source(cls):
         'def from_dict(cls, o):',
         '    args = []',
     ]
+
+    fields_by_name = {f.name: f for f in dataclass_fields(cls)}
+
     for name, field_type in typing.get_type_hints(cls).items():
 
         # pop off the top layer of optional, since we are using o.get
         if typing.get_origin(field_type) == typing.Union:
             field_type = typing.get_args(field_type)[0]
 
-        access = f'o.get({name!r})'
+        field = fields_by_name[name]
+
+        input_name = name
+        if has_meta(field, 'field_name'):
+            input_name = field.metadata['fastclasses_json']['field_name']
+            if not isinstance(input_name, str):
+                raise TypeError(
+                    "fastclasses_json, field_name must be str: "
+                    f"{cls.__name__}.{name}"
+                )
+
+        access = f'o.get({input_name!r})'
 
         transform = expr_builder_from(field_type)
+        if has_meta(field, 'decoder'):
+            transform = decoder_expr(name)
 
         if transform('x') != 'x':
             lines.append(f'    value = {access}')
@@ -147,12 +172,29 @@ def _to_dict_source(cls):
     # TODO: option for including Nones or not
     INCLUDE_NONES = False
 
+    fields_by_name = {f.name: f for f in dataclass_fields(cls)}
+
     for name, field_type in typing.get_type_hints(cls).items():
 
         access = f'self.{name}'
 
         transform = expr_builder_to(field_type)
 
+        # custom encoder and decoder routines
+        field = fields_by_name[name]
+        if has_meta(field, 'encoder'):
+            transform = encoder_expr(name)
+
+        # custom mapping of dataclass fieldnames to json field names
+        output_name = name
+        if has_meta(field, 'field_name'):
+            output_name = field.metadata['fastclasses_json']['field_name']
+            if not isinstance(output_name, str):
+                raise TypeError(
+                    "fastclasses_json, field_name must be str: "
+                    f"{cls.__name__}.{name}"
+                )
+
         if transform('x') != 'x':
             # since we have an is not none check, elide the first level
             # of optional
@@ -162,17 +204,51 @@ def _to_dict_source(cls):
             lines.append(f'    if value is not None:')  # noqa: F541
             lines.append(f'        value = ' + transform('value'))  # noqa: E501,F541
             if INCLUDE_NONES:
-                lines.append(f'    result[{name!r}] = value')
+                lines.append(f'    result[{output_name!r}] = value')
             else:
-                lines.append(f'        result[{name!r}] = value')
+                lines.append(f'        result[{output_name!r}] = value')
         else:
-            lines.append(f'    result[{name!r}] = {access}')
+            lines.append(f'    result[{output_name!r}] = {access}')
 
     lines.append('    return result')
     lines.append('')
     return '\n'.join(lines)
 
 
+def encoders(cls):
+    result = {}
+    for field in dataclass_fields(cls):
+        if has_meta(field, 'encoder'):
+            sym = f'{field.name}#encoder'
+            result[sym] = field.metadata['fastclasses_json']['encoder']
+    return result
+
+
+def decoders(cls):
+    result = {}
+    for field in dataclass_fields(cls):
+        if has_meta(field, 'decoder'):
+            sym = f'{field.name}#decoder'
+            result[sym] = field.metadata['fastclasses_json']['decoder']
+    return result
+
+
+def has_meta(field, meta):
+    if field.metadata and field.metadata.get('fastclasses_json', {}).get(meta):
+        return True
+    return False
+
+
+def encoder_expr(name):
+    # this funny x#encoder will have been placed in globals
+    return lambda expr: f'globals()["{name}#encoder"]({expr})'
+
+
+def decoder_expr(name):
+    # this funny x#decoder will have been placed in globals
+    return lambda expr: f'globals()["{name}#decoder"]({expr})'
+
+
 def expr_builder_from(t: type, depth=0):
     return expr_builder(t, depth, direction=_FROM)
 
